#include "Systray.hpp"
#include "NotificationArea.hpp"
#include <QtGui/QIcon>
#include <QtGui/QPixmap>
#include <QtGui/QPainter>
#include <QtWidgets/QMenu>

//=============================================================================

Systray::Systray( QObject* parent )
    : QObject( parent )
    , m_Systray( NULL )
    , m_NotificationArea( NULL )
    , m_actShow( NULL )
    , m_actExit( NULL )
{
    m_Systray = new QSystemTrayIcon( createSystrayIcon() );
    m_Systray->setToolTip( tr( "Resolution Test" ) );

    connect( m_Systray, SIGNAL( activated( QSystemTrayIcon::ActivationReason ) ), this, SLOT( onSystrayActivated( QSystemTrayIcon::ActivationReason ) ) );

    m_NotificationArea = new NotificationArea();
    m_NotificationArea->setVisible( false );

    m_actShow = new QAction( tr( "Show Notifications" ), this );
    connect( m_actShow, SIGNAL( triggered() ), this, SLOT( showNotificationArea() ) );

    m_actExit = new QAction( tr( "Exit" ), this );
    connect( m_actExit, SIGNAL( triggered() ), this, SLOT( onExit() ) );

    QMenu *menu = new QMenu();
    menu->addAction( m_actShow );
    menu->addSeparator();
    menu->addAction( m_actExit );
    m_Systray->setContextMenu( menu );
}

Systray::~Systray()
{
    delete m_Systray;
    m_Systray = NULL;

    delete m_NotificationArea;
    m_NotificationArea = NULL;
}

void Systray::start()
{
    m_Systray->show();
}

void Systray::showNotificationArea()
{
    m_NotificationArea->show();
}

void Systray::toggleShowNotificationArea()
{
    m_NotificationArea->setVisible( !m_NotificationArea->isVisible() );
}

void Systray::onSystrayActivated( QSystemTrayIcon::ActivationReason reason )
{
    switch ( reason )
    {
    case QSystemTrayIcon::ActivationReason::Trigger:
        toggleShowNotificationArea();
        break;
    }
}

void Systray::onExit()
{
    emit Exit();
}

QIcon Systray::createSystrayIcon() const
{
    // dummy icon for systray
    QIcon icon;
    QSize pxmapSize( 16, 16 );
    QPixmap pxmap( pxmapSize );
    pxmap.fill( Qt::transparent );
    QPainter p( &pxmap );
    p.setRenderHint( QPainter::Antialiasing );
    p.setBrush( QColor( 0x5db758 ) );
    p.setPen( QColor( 0, 0, 0, 0 ) );
    p.drawEllipse( QRect( QPoint(), pxmapSize ) );
    icon.addPixmap( pxmap );
    return  icon;
}

//=============================================================================